local addon = PvPTimer
local L = addon.Locale
local LSM = addon.Lib.Media

local CopyTable = addon.CopyTable

-- generic getter/setter functions, use AceGUI's IDs to get/set values
local function getConfigValue(i)
	local db = addon.DB.profile
	
	for k = 2, #i do
		if not db then db = {} end
		db = db[i[k]]
	end

	return db
end

local function setConfigValue(i, v)
	local db = addon.DB.profile
	
	for k = 2, #i-1 do
		if not db[i[k]] then db[i[k]] = {} end
		db = db[i[k]]
	end
	
	db[i[#i]] = v
end

-- for colors
local function getColorValue(i)
	local t = getConfigValue(i)
	
	if t then
		return t.r, t.g, t.b, t.a
	else
		return 0, 0, 0, 1
	end
end

local function setColorValue(i, r, g, b, a)
	local t = {
		["r"] = r,
		["g"] = g,
		["b"] = b,
		["a"] = a,
	}
	setConfigValue(i, t)
end

-- for multiselect controls
local function getMultiValue(i, k)
	i[#i+1] = k
	return getConfigValue(i)
end

local function setMultiValue(i, k, v)
	i[#i+1] = k
	setConfigValue(i, v)
end

local const_bg = addon.Const.Battlegrounds
local const_arena = addon.Const.Arenas
local const_autohide = addon.Const.Autohide
local const_alerttarget = addon.Const.AlertTarget
local const_attach = addon.Const.Attach
local const_justify = addon.Const.Justify
local const_startpoint = addon.Const.StartPoint

-- option subtables
local alert = {
	CautionHeader = {
		order = 1,
		name = L["CAUTION"],
		type = "header",
	},
	CautionText = {
		order = 2,
		name = L["Be careful when using these alerts, they can send out a lot of messages. People will likely yell at you or /ignore you if you spam alerts in a public channel."],
		type = "description",
		width = "full",
		fontSize = "medium",
	},
	EnabledHeader = {
		order = 10,
		name = L["Enable Alert"],
		type = "header",
	},
	Enabled = {
		order = 11,
		name = L["Enable alerts"],
		desc = L["Enable or disable alerts regardless of other settings."],
		type = "toggle",
	},
	ZoneText = {
		order = 20,
		name = L["Alert Target"],
		type = "header",
	},
	BG = {
		order = 21,
		name = L["Battlegrounds"],
		desc = L["Enable or disable in normal, non-rated Battlegrounds."],
		type = "select",
		values = const_alerttarget,
	},
	BG_OnlyTarget = {
		order = 22,
		name = L["Only target/focus"],
		desc = L["Only trigger alerts if source is the current target or focus."],
		type = "toggle",
	},
	RatedBG = {
		order = 31,
		name = L["Rated Battlegrounds"],
		desc = L["Enable or disable in rated Battlegrounds."],
		type = "select",
		values = const_alerttarget,
	},
	RatedBG_OnlyTarget = {
		order = 32,
		name = L["Only target/focus"],
		desc = L["Only trigger alerts if source is the current target or focus."],
		type = "toggle",
	},
	Arena = {
		order = 41,
		name = "Arenas",
		desc = L["Enable or disable in arenas."],
		type = "select",
		values = const_alerttarget,
	},
	Arena_OnlyTarget = {
		order = 42,
		name = L["Only target/focus"],
		desc = L["Only trigger alerts if source is the current target or focus."],
		type = "toggle",
	},
	Else = {
		order = 98,
		name = L["Everywhere else"],
		desc = L["Enable or disable anywhere else not specified above."],
		type = "select",
		values = const_alerttarget,
	},
	Else_OnlyTarget = {
		order = 99,
		name = L["Only target/focus"],
		desc = L["Only trigger alerts if source is the current target or focus."],
		type = "toggle",
	},
}

local anchor = {
	Enabled = {
		order = 1,
		name = L["Enable anchor"],
		desc = L["Enables or disables this anchor."],
		type = "toggle",
		set = function(i, v)
			setConfigValue(i, v)
			addon:RefreshAnchors()
		end,
		width = "full",
	},
	Width = {
		order = 11,
		name = L["Width"],
		desc = L["Width of the anchor."],
		type = "range",
		min = 100,
		max = 400,
		step = 5,
		width = "full",
	},
	Scale = {
		order = 12,
		name = L["Scale"],
		desc = L["Scale of the anchor."],
		type = "range",
		min = 0.1,
		max = 5,
		step = 0.05,
		isPercent = true,
		width = "full",
	},
	Text_Spec = {
		order = 20,
		name = L["Talent specialization icon"],
		type = "header",
	},
	ShowSpec = {
		order = 21,
		name = L["Spec icon size"],
		desc = L["Shows an icon that displays the player's talent specialization. Set to 0 to disable."],
		type = "range",
		min = 0,
		softMax = 64,
		step = 1,
	},
	SpecPosition = {
		order = 22,
		name = L["Position"],
		desc = L["Position of the icon."],
		type = "select",
		values = addon.Const.IconPosition,
	},
	Text_Enable = {
		order = 50,
		name = L["Show anchor in"],
		type = "header",
		width = "full",
	},
	BG = {
		order = 51,
		name = L["Battlegrounds"],
		desc = L["Enable or disable in normal, non-rated Battlegrounds."],
		type = "toggle",
	},
	RatedBG = {
		order = 52,
		name = L["Rated Battlegrounds"],
		desc = L["Enable or disable in rated Battlegrounds."],
		type = "toggle",
	},
	Arena = {
		order = 53,
		name = L["Arenas"],
		desc = L["Enable or disable in arenas."],
		type = "toggle",
	},
	-- Duel = {
		-- order = 54,
		-- name = "Duels",
		-- type = "toggle",
	-- },
	-- PvP = {
		-- order = 55,
		-- name = "PvP flagged",
		-- type = "toggle",
	-- },
	Else = {
		order = 59,
		name = L["Everywhere else"],
		desc = L["Enable or disable anywhere else not specified above."],
		type = "toggle",
	},
	Position = {
		order = 10,
		name = L["Position"],
		desc = L["Position of the anchor."],
		type = "group",
		args = {
			AttachPoint = {
				order = 11,
				name = L["Anchor point"],
				desc = L["Sets the point that will be anchored."],
				type = "select",
				values = const_attach,
			},
			AttachFrame = {
				order = 12,
				name = L["Relative to"],
				desc = L["Sets the frame to be anchored to.\nUse 'UIParent' for the whole screen."],
				type = "input",
				set = function(i, v)
					if not _G[v] or not _G[v].SetPoint then
						v = "UIParent"
						addon:Print(L["Invalid frame. Resetting to default."])
					end
					setConfigValue(i, v)
					addon:RefreshAnchors()
				end,
			},
			AttachRelative = {
				order = 13,
				name = L["Relative point"],
				desc = L["Sets the point to be anchored to."],
				type = "select",
				values = const_attach,
			},
			AttachX = {
				order = 14,
				name = L["Offset X"],
				desc = L["Horizontal offset."],
				type = "range",
				softMin = -1000,
				softMax = 1000,
				min = -2000,
				max = 2000,
				step = 1,
				bigStep = 5,
				width = "full",
			},
			AttachY = {
				order = 15,
				name = L["Offset Y"],
				desc = L["Vertical offset."],
				type = "range",
				softMin = -1000,
				softMax = 1000,
				min = -2000,
				max = 2000,
				step = 1,
				bigStep = 5,
				width = "full",
			},
		},
	},
	Bars = {
		order = 30,
		name = L["Timer bars"],
		desc = L["Timer bar settings."],
		type = "group",
		args = {
			-- CopySettings = {
				-- order = 1,
				-- name = "Copy settings",
				-- type = "execute",
			-- },
			-- PasteSettings = {
				-- order = 2,
				-- name = "Paste settings",
				-- type = "execute",
				-- disabled = function()
					-- if addon.CopyBarSettings == nil then
						-- return true
					-- else
						-- return false
					-- end
				-- end,
			-- },
			IconMode = {
				order = 3,
				name = L["Icon Mode"],
				desc = L["Show icons instead of bars."],
				type = "toggle",
			},
			IconSize = {
				order = 4,
				name = L["Icon Size"],
				desc = L["Size of the icons."],
				type = "range",
				min = 0,
				softMin = 8,
				softMax = 64,
				step = 1,
			},
			Color = {
				order = 11,
				name = L["Bar color"],
				desc = L["Default timer bar color."],
				type = "color",
				hasAlpha = true,
				get = getColorValue,
				set = function(i, r, g, b, a)
					setColorValue(i, r, g, b, a)
					addon:RefreshAnchors()
				end,
			},
			TypeColor = {
				order = 12,
				name = L["Use spell category color"],
				desc = L["Use spell category color (defined in Spell Categories) instead."],
				type = "toggle",
			},
			Texture = {
				order = 21,
				name = L["Bar texture"],
				desc = L["Texture of timer bars."],
				type = "select",
				dialogControl = 'LSM30_Statusbar',
				values = AceGUIWidgetLSMlists.statusbar,
				width = "full",
			},
			StartPoint = {
				order = 31,
				name = L["Bar start point"],
				desc = L["Set the corner and side bars start from."],
				type = "select",
				values = const_startpoint,
			},
			Columns = {
				order = 32,
				name = L["Columns"],
				desc = L["Arrange bars into columns. (0: automatic)"],
				type = "range",
				min = 0,
				max = 20,
				step = 1,
			},
			MaxBars = {
				order = 33,
				name = L["Visible Bars"],
				desc = L["Limit the number of visible bars. (0: no limit)"],
				type = "range",
				min = 0,
				max = 20,
				step = 1,
			},
			Text_SpellName = {
				order = 40,
				name = L["Spell Name"],
				type = "header",
			},
			NameFont = {
				order = 41,
				name = L["Font"],
				type = "select",
				dialogControl = 'LSM30_Font',
				values = AceGUIWidgetLSMlists.font,
			},
			NameFontSize = {
				order = 42,
				name = L["Font size"],
				type = "range",
				min = 1,
				max = 20,
				step = 1,
			},
			Text_Time = {
				order = 50,
				name = L["Time Remaining"],
				type = "header",
			},
			TimeFont = {
				order = 51,
				name = L["Font"],
				type = "select",
				dialogControl = 'LSM30_Font',
				values = AceGUIWidgetLSMlists.font,
			},
			TimeFontSize = {
				order = 52,
				name = L["Font size"],
				type = "range",
				min = 1,
				max = 20,
				step = 1,
			},
			Text_Ghost = {
				order = 60,
				name = L["Ghost"],
				type = "header",
			},
			Ghost = {
				order = 61,
				name = L["Duration"],
				desc = L["Sets the duration that the timer persists after expiring as a ghost. Set to 0 to disable."],
				type = "range",
				min = 0,
				softMax = 10,
				step = 0.1,
			},
			GhostBackground = {
				order = 62,
				type = "color",
				name = L["Background color"],
				desc = L["Changes the background color when the effect begins. Does not apply in icon mode."],
				get = getColorValue,
				set = function(i, r, g, b, a)
					setColorValue(i, r, g, b, a)
					addon:RefreshAnchors()
				end,
			},
			GhostMessage = {
				order = 63,
				type = "input",
				name = L["Message"],
				desc = L["Show a message in place of the bar's timer."],
			},
			Text_Fadeout = {
				order = 70,
				name = L["Fade out"],
				type = "header",
			},
			Fadeout = {
				order = 71,
				name = L["Length"],
				desc = L["Sets the amount of time the timer takes to fade out when it finishes. Set to 0 to disable."],
				type = "range",
				min = 0,
				softMax = 10,
				step = 0.1,
			},
			FadeoutBackground = {
				order = 72,
				type = "color",
				name = L["Background color"],
				desc = L["Changes the background color when the effect begins. Does not apply in icon mode."],
				get = getColorValue,
				set = function(i, r, g, b, a)
					setColorValue(i, r, g, b, a)
					addon:RefreshAnchors()
				end,
			},
			FadeoutMessage = {
				order = 73,
				type = "input",
				name = L["Message"],
				desc = L["Show a message in place of the bar's timer."],
			},
		},
	},
}

local anchor_arena = addon:CopyTable(anchor)
anchor_arena.Autohide = nil
anchor_arena.Text_Enable = nil
anchor_arena.BG = nil
anchor_arena.RatedBG = nil
anchor_arena.Arena = nil
anchor_arena.Duel = nil
anchor_arena.PvP = nil
anchor_arena.Else = nil

anchor_arena.Position.args.NextX = {
	order = 18,
	name = L["Distance between frames X"],
	desc = L["Horizontal distance between each arena frame."],
	type = "range",
	softMin = -1000,
	softMax = 1000,
	min = -2000,
	max = 2000,
	step = 1,
	bigStep = 5,
	width = "full",
}
anchor_arena.Position.args.NextY = {
	order = 19,
	name = L["Distance between frames Y"],
	desc = L["Vertical distance between each arena frame."],
	type = "range",
	softMin = -1000,
	softMax = 1000,
	min = -2000,
	max = 2000,
	step = 1,
	bigStep = 5,
	width = "full",
}

local spellcat = {
	offensive = {
		order = 101,
		type = "color",
		name = L["Offensive CDs"],
		desc = L["Cooldowns that increase damage output, e.g. Recklessness."],
		get = getColorValue,
		set = function(i, r, g, b, a)
			setColorValue(i, r, g, b, a)
			addon:RefreshAnchors()
		end,
	},
	defensive = {
		order = 102,
		type = "color",
		name = L["Defensive CDs"],
		desc = L["Defensive and healing cooldowns, e.g. Shield Wall or Swiftmend."],
		get = getColorValue,
		set = function(i, r, g, b, a)
			setColorValue(i, r, g, b, a)
			addon:RefreshAnchors()
		end,
	},
	interrupt = {
		order = 103,
		type = "color",
		name = L["Interrupts and Silences"],
		desc = L["Abilities that interrupt spellcasting or silence, e.g. Kick."],
		get = getColorValue,
		set = function(i, r, g, b, a)
			setColorValue(i, r, g, b, a)
			addon:RefreshAnchors()
		end,
	},
	cc = {
		order = 104,
		type = "color",
		name = L["Crowd Control"],
		desc = L["Abilities that make you lose control of your character, e.g. Fear or Polymorph."],
		get = getColorValue,
		set = function(i, r, g, b, a)
			setColorValue(i, r, g, b, a)
			addon:RefreshAnchors()
		end,
	},
	root = {
		order = 105,
		type = "color",
		name = L["Roots and Snares"],
		desc = L["Spells that restrict or prevent movement, e.g. Frost Nova or Hamstring."],
		get = getColorValue,
		set = function(i, r, g, b, a)
			setColorValue(i, r, g, b, a)
			addon:RefreshAnchors()
		end,
	},
	misc = {
		order = 106,
		type = "color",
		name = L["Miscellaneous Spells"],
		desc = L["Anything else. Spells that don't really fit into other categories."],
		get = getColorValue,
		set = function(i, r, g, b, a)
			setColorValue(i, r, g, b, a)
			addon:RefreshAnchors()
		end,
	},
}

addon.EmptyOptions = {
	type = "group",
	name = "PvPTimer",
	args = {
	},
}

--main option table
addon.Options = {
	type = "group",
	name = "PvPTimer",
	get = getConfigValue,
	set = setConfigValue,
	args = {
		Settings = {
			order = 1,
			name = L["Settings"],
			type = "group",
			args = {
--				EnableAddon = {
--					order = 1,
--					name = L["Enable addon"],
--					desc = L["Enables or disables the whole addon."],
--					type = "toggle",
--					set = function(i, v)
--						setConfigValue(i, v)
--						addon:ZONE_CHANGED_NEW_AREA();
--					end,
--				},
				Locked = {
					order = 2,
					name = function()
						if addon.AnchorsLocked then
							return L["Unlock anchors"]
						else
							return L["Lock anchors"]
						end
					end,
					desc = L["Locks or unlocks anchors so they can be dragged around and resized using the mouse."],
					type = "execute",
					func = function() addon:ToggleLock() end,
				},
				Test = {
					order = 3,
					name = L["Run Test"],
					desc = L["Shows some test bars."],
					type = "execute",
					func = function() addon:RunTest() end,
				},
				OpenSpellConfig = {
					order = 4,
					name = L["Spell Configuration"],
					desc = L["Opens the Spell Configuration window, where you can disable and adjust spells."],
					type = "execute",
					func = function() addon:ToggleSpellConfig() end,
				},
				Anchors = {
					order = 11,
					name = L["Anchor Settings"],
					desc = L["Anchors are locations on screen to which timers are added."],
					type = "group",
					set = function(i, v)
						setConfigValue(i, v)
						addon:RefreshAnchors()
					end,
					args = {
						Anchor_Target = {
							order = 2,
							name = L["Target"],
							desc = L["Anchor that shows information about your current target."],
							type = "group",
							args = anchor,
						},
						Anchor_Focus = {
							order = 3,
							name = L["Focus"],
							desc = L["Anchor that shows information about your current focus target."],
							type = "group",
							args = anchor,
						},
						Anchor_Arena = {
							order = 4,
							name = L["Arena Targets"],
							desc = L["Anchor that shows information about arena opponents."],
							type = "group",
							args = anchor_arena,
						},
					},
				},
				Alert = {
					order = 21,
					name = L["Alert Messages"],
					desc = L["The addon can send out messages when spell cooldowns are used/ready/etc."],
					type = "group",
					args = {
						offensive = {
							order = 101,
							type = "group",
							name = L["Offensive CDs"],
							desc = L["Cooldowns that increase damage output, e.g. Recklessness."],
							args = alert,
						},
						defensive = {
							order = 102,
							type = "group",
							name = L["Defensive CDs"],
							desc = L["Defensive and healing cooldowns, e.g. Shield Wall or Swiftmend."],
							args = alert,
						},
						interrupt = {
							order = 103,
							type = "group",
							name = L["Interrupts and Silences"],
							desc = L["Abilities that interrupt spellcasting or silence, e.g. Kick."],
							args = alert,
						},
						cc = {
							order = 104,
							type = "group",
							name = L["Crowd Control"],
							desc = L["Abilities that make you lose control of your character, e.g. Fear or Polymorph."],
							args = alert,
						},
						root = {
							order = 105,
							type = "group",
							name = L["Roots and Snares"],
							desc = L["Spells that restrict or prevent movement, e.g. Frost Nova or Hamstring."],
							args = alert,
						},
						misc = {
							order = 106,
							type = "group",
							name = L["Miscellaneous Spells"],
							desc = L["Anything else. Spells that don't really fit into other categories."],
							args = alert,
						},
						spec = {
							order = 999,
							type = "group",
							name = L["Spec Detection"],
							desc = L["Talent spec detection alerts."],
							args = alert,
						},
					},
				},
				SpellCategories = {
					order = 31,
					name = L["Spell Categories"],
					desc = L["Spell cooldowns are organized into categories."],
					type = "group",
					args = spellcat,
				},
			},
		},
	},
}
