--@author Zardas
--@release API for creating and managing timer bar groups.
--Use lib:NewGroup to create a new group.
--All parameters are optional, use nil or just leave them out to omit them.

-- Default settings
local def_Width = 200
local def_MinWidth = 100
local def_MaxWidth = 1600
local def_StartPoint = "bottomright"
local def_MaxBars = 0
local def_Columns = 0
local def_MinColumns = 0
local def_MaxColumns = 50
--local def_AutoResize = true
local def_SortField = "time"
local def_SortAscending = false
local def_IconMode = false
local def_IconSize = 32

-- Set up libraries
local lib = LibStub("LibFancyBar-1.0")
local cbh = LibStub:GetLibrary("CallbackHandler-1.0")

-- Copy global functions to local
local GetTime = GetTime
local CreateFrame = CreateFrame
local setmetatable = setmetatable
local tremove = table.remove
local tsort = table.sort
local floor = floor
local unpack = unpack
local pairs = pairs
local ipairs = ipairs
local GT = GameTooltip
local mrand = math.random
local wipe = wipe
local GetSpellInfo = GetSpellInfo
local select = select
local hooksecurefunc = hooksecurefunc

local cfg = {
	"Columns",
	"Visible bars",
--	"Resize bars",
	"Bar starting point",
--	"Sorting"
--	"Icon Mode",
}

local startpoint = {
	["bottomright"] = "Below on right",
	["bottomleft"] = "Below on left",
	["topright"] = "Above on right",
	["topleft"] = "Above on left"
}

local sortfield = {
	"Time remaining",
	"Label (alphabetically)",
	"Time created"
}

local testicons = {
	49028, 49206, 47528, 77606, 91802, 49184, 5217, 5211, 29166, 18562, 48438, 5384, 82945,
	19236, 89485, 88685, 586, 88684, 6229, 6789, 6360, 71521, 17877, 2565, 6544, 5246, 50720,
	498, 96231, 20066, 20925
}

-- Create a dummy frame to store inherited functions.
-- Idea from LibBars-1.0 and LibCandyBar-3.0
lib.groupPrototype = lib.groupPrototype or CreateFrame("Frame")
lib.groupPrototype_mt = lib.groupPrototype_mt or {__index = lib.groupPrototype}

local groupPrototype = lib.groupPrototype
local groupPrototype_mt = lib.groupPrototype_mt

-- Dragging and resizing functions
local function onDragHandleMouseDown(self)
	self:GetParent():StartSizing("BOTTOMRIGHT")
end

local function onDragHandleMouseUp(self, button)
	self:GetParent():StopMovingOrSizing()
end

local function onResize(self, width)
	self.width = width

	self.FG_ConfigCombo:ClearAllPoints()
	if width < 330+self.FG_HeaderText:GetStringWidth() then
		self.FG_ConfigCombo:SetPoint("BOTTOMLEFT", self, "TOPLEFT", -15, 2)
	else
		self.FG_ConfigCombo:SetPoint("BOTTOMLEFT", self, "BOTTOMLEFT", -15, -2)
	end
	self.callbacks:Fire("FancyBar_GroupResize", self, width)
	self:Rearrange()
end

local function onDragStart(self)
	self:StartMoving()
end

local function onDragStop(self)
	self:StopMovingOrSizing()

	local p, f, rp, x, y = self:GetPoint()
	self.callbacks:Fire("FancyBar_GroupMove", self, p, f, rp, x, y)
end

local function onSetScale(self, scale)
	for k in pairs(self.bars) do
		k:SetScale(scale)
	end
end

-- shows the config element chosen in combo box
local function Config_ShowElement(group, hide)
	group.FG_ConfigColumns:Hide()
	group.FG_ConfigMaxBars:Hide()
--	group.FG_ConfigResize:Hide()
	group.FG_ConfigStartPoint:Hide()
--	group.FG_ConfigIconMode:Hide()

	if hide or not group.FG_ConfigCombo:IsShown() then return end

	local id = UIDropDownMenu_GetSelectedID(group.FG_ConfigCombo)

	if id == 1 then
		group.FG_ConfigColumns:Show()
	elseif id == 2 then
		group.FG_ConfigMaxBars:Show()
--	elseif id == 3 then
--		group.FG_ConfigResize:Show()
	elseif id == 3 then
		group.FG_ConfigStartPoint:Show()
--	elseif id == 4 then
--		group.FG_ConfigIconMode:Show()
	end
end

-- config combo box value changed
local function Config_OnClick(self, group)
	UIDropDownMenu_SetSelectedValue(group.FG_ConfigCombo, self.value)
	Config_ShowElement(group)
end

-- config startpoint value changed
local function Config_StartPoint(self, group)
	UIDropDownMenu_SetSelectedValue(group.FG_ConfigStartPoint, self.value)
	group:SetStartPoint(self.value)
end

-- config combo box init
local function Config_Initialize(self, level, items, func, group)
	local info
	for k, v in pairs(items) do
		info = UIDropDownMenu_CreateInfo()
		info.text = v
		info.value = k
		info.func = func
		UIDropDownMenu_AddButton(info, level)
	end
end

-- shows a tooltip for header buttons
local function ShowTooltip(obj, header, text)
	GT:SetOwner(obj, "ANCHOR_BOTTOMLEFT", -2, -2)
	GT:ClearLines()
	GT:AddLine(header)
	GT:AddLine(" ")
	GT:AddLine(text, 1, 1, 1, true)
	GT:Show()
end

-- Shows/hides the config elements of a group's header.
local function ToggleConfig(group)
	if not group and not group.FG_Header then return end

	if group.FG_ConfigCombo:IsShown() then
		group.FG_ConfigCombo:Hide()
		Config_ShowElement(group, true)
	else
		group.FG_ConfigCombo:Show()
		Config_ShowElement(group)
	end
end

-- Bar sorting function
local function SortBars(a, b, field)
	-- sort using the time remaining
	if field == "time" then
		return (a.remaining < b.remaining)
	-- sort using the bar's label
	elseif field == "label" then
		return (a.labeltext < b.labeltext)
	-- no sorting, uses the bar's creation time
	elseif field == "created" then
		return (a.created < b.created)
	-- sort using a data field
	else
		return (a:GetData(field) < b:GetData(field))
	end
end

--- Creates a new timerbar group and returns it.
--@param width Width of the group in pixels.
--Default: 200.
--@param title Title of the group. This is shown when the group is unlocked for moving.
--@param name Name of the group. This is used to access the group.
function lib:NewGroup(width, title, name)
	local f = CreateFrame("Frame", name, UIParent)
	f:SetWidth(width or def_Width)
	f:SetHeight(28)
	f:SetMinResize(def_MinWidth, f:GetHeight())
	f:SetMaxResize(def_MaxWidth, f:GetHeight())
	f:SetClampedToScreen(true)

	-- import prototype functions
	local group = setmetatable(f, groupPrototype_mt)

	local bg = group:CreateTexture(nil, "PARENT")
	bg:SetAllPoints(group)
	bg:SetBlendMode("BLEND")
	bg:SetTexture(0, 0, 0, 0.5)
	group.FG_Header = bg

	local drag = CreateFrame("Frame", nil, group)
	drag:SetFrameLevel(group:GetFrameLevel() + 10)
	drag:SetWidth(16)
	drag:SetHeight(16)
	drag:SetPoint("BOTTOMRIGHT", bg)
	drag:EnableMouse(true)
	drag:SetScript("OnMouseDown", onDragHandleMouseDown)
	drag:SetScript("OnMouseUp", onDragHandleMouseUp)
	group.FG_DragArea = drag

	local tex = drag:CreateTexture(nil, "BACKGROUND")
	tex:SetBlendMode("ADD")
	tex:SetTexture("Interface\\Addons\\LibFancyBar-1.0\\draghandle")
	tex:SetPoint("BOTTOMRIGHT", drag, "BOTTOMRIGHT")
	group.FG_DragHandle = tex

--[[
	local lockbtn = CreateFrame("Button", nil, group, "UIPanelButtonGrayTemplate")
	lockbtn:SetText("X")
	lockbtn:SetHeight(24)
	lockbtn:SetPoint("LEFT", bg, "RIGHT", -36, -2)
	lockbtn:SetPoint("RIGHT", bg, "RIGHT", -16, -2)
	lockbtn:RegisterForClicks("LeftButtonUp")
	lockbtn:SetScript("OnClick", function() group:Lock() end)
	lockbtn:SetScript("OnEnter", function() ShowTooltip(lockbtn, "Lock Anchor", "Click here to lock this anchor.") end)
	lockbtn:SetScript("OnLeave", function() GT:Hide() end)
	group.FG_LockButton = lockbtn
]]

	local configbtn = CreateFrame("Button", nil, group, "UIPanelButtonGrayTemplate")
--	configbtn:SetPoint("RIGHT", lockbtn, "LEFT")
	configbtn:SetPoint("LEFT", bg, "RIGHT", -36, -2)
	configbtn:SetPoint("RIGHT", bg, "RIGHT", -16, -2)
	configbtn:SetText("C")
	configbtn:SetHeight(24)
	configbtn:SetWidth(20)
	configbtn:RegisterForClicks("LeftButtonUp")
	configbtn:SetScript("OnClick", function() ToggleConfig(group) end)
	configbtn:SetScript("OnEnter", function() ShowTooltip(configbtn, "Show Configuration Settings", "Click here to show/hide this anchor's configuration settings.") end)
	configbtn:SetScript("OnLeave", function() GT:Hide() end)
	group.FG_ConfigButton = configbtn

	local testbtn = CreateFrame("Button", nil, group, "UIPanelButtonGrayTemplate")
	testbtn:SetPoint("RIGHT", configbtn, "LEFT", -2)
	testbtn:SetText("T")
	testbtn:SetHeight(24)
	testbtn:SetWidth(20)
	testbtn:RegisterForClicks("LeftButtonUp")
	testbtn:SetScript("OnClick", function()
		if IsControlKeyDown() then group:StopTest()
		elseif IsShiftKeyDown() then group:RunTest(50)
		else group:RunTest(20)
		end
	end)
	testbtn:SetScript("OnEnter", function() ShowTooltip(testbtn, "Create Test Bars", "Click here to show 20 test bars.\nShift-Click for 50.\n\nCtrl-Click to stop the test.") end)
	testbtn:SetScript("OnLeave", function() GT:Hide() end)
	group.FG_TestButton = testbtn

	local text = group:CreateFontString(nil, "OVERLAY")
	text:SetFontObject(GameFontNormal)
	text:SetText(title or "")
	text:SetAllPoints(group)
	text:SetPoint("RIGHT", testbtn, "LEFT", 0, -2)
	text:SetJustifyH("RIGHT")
	text:SetJustifyV("MIDDLE")
	group.FG_HeaderText = text

	local comboname = tostring(group).."_Config"
	CreateFrame("Button", comboname, group, "UIDropDownMenuTemplate")
	local combo = _G[comboname]
	combo:SetPoint("LEFT", group, "LEFT", -15, -2)
	local func = function(self) Config_OnClick(self, group) end
	UIDropDownMenu_Initialize(combo, function(self, level) Config_Initialize(self, level, cfg, func, group) end)
	UIDropDownMenu_SetWidth(combo, 120)
	UIDropDownMenu_SetButtonWidth(combo, 124)
	UIDropDownMenu_SetSelectedID(combo, 1)
	UIDropDownMenu_JustifyText(combo, "LEFT")
	group.FG_ConfigCombo = combo

	local cfg_columns = CreateFrame("Slider", "FB_"..tostring(group).."_ConfigColumns", group, "OptionsSliderTemplate")
	cfg_columns:SetMinMaxValues(def_MinColumns, def_MaxColumns)
	cfg_columns:SetWidth(100)
	cfg_columns:SetValueStep(1)
	cfg_columns:EnableMouseWheel(true)
	cfg_columns.Low = _G[cfg_columns:GetName().."Low"]
	cfg_columns.High = _G[cfg_columns:GetName().."High"]
	cfg_columns.Value = cfg_columns:CreateFontString(nil, 'ARTWORK', 'GameFontWhite')
	cfg_columns.Value:SetPoint("BOTTOM", 0, -10)
	cfg_columns.Value:SetWidth(50)
	cfg_columns:SetScript("OnValueChanged", function() group:SetColumns(floor(cfg_columns:GetValue())) end)
	cfg_columns.Low:SetText(def_MinColumns)
	cfg_columns.High:SetText(def_MaxColumns)
	cfg_columns:SetPoint("LEFT", combo, "RIGHT", -8, 6)
	group.FG_ConfigColumns = cfg_columns

	local cfg_maxbars = CreateFrame("Slider", "FB_"..tostring(group).."_ConfigMaxBars", group, "OptionsSliderTemplate")
	cfg_maxbars:SetMinMaxValues(0, 50)
	cfg_maxbars:SetWidth(100)
	cfg_maxbars:SetValueStep(1)
	cfg_maxbars:EnableMouseWheel(true)
	cfg_maxbars.Low = _G[cfg_maxbars:GetName().."Low"]
	cfg_maxbars.High = _G[cfg_maxbars:GetName().."High"]
	cfg_maxbars.Value = cfg_maxbars:CreateFontString(nil, 'ARTWORK', 'GameFontWhite')
	cfg_maxbars.Value:SetPoint("BOTTOM", 0, -10)
	cfg_maxbars.Value:SetWidth(50)
	cfg_maxbars:SetScript("OnValueChanged", function() group:SetVisibleBars(floor(cfg_maxbars:GetValue())) end)
	cfg_maxbars.Low:SetText(0)
	cfg_maxbars.High:SetText(50)
	cfg_maxbars:SetPoint("LEFT", combo, "RIGHT", -8, 6)
	group.FG_ConfigMaxBars = cfg_maxbars

--[[
	local cfg_resize = CreateFrame("CheckButton", "FB_"..tostring(group).."_ConfigResize", group, "UICheckButtonTemplate")
	cfg_resize.Text = _G[cfg_resize:GetName().."Text"]
	cfg_resize.Text:SetText("Enabled")
	cfg_resize:SetScript("OnClick", function() group:SetAutoResize(cfg_resize:GetChecked()) end)
	cfg_resize:SetPoint("LEFT", combo, "RIGHT", -15, 0)
	group.FG_ConfigResize = cfg_resize

]]
	local cfg_startpoint = tostring(group).."_ConfigStartPoint"
	CreateFrame("Button", cfg_startpoint, group, "UIDropDownMenuTemplate")
	_G[cfg_startpoint]:SetPoint("LEFT", combo, "RIGHT", -28, 0)
	local func = function(self) Config_StartPoint(self, group) end
	UIDropDownMenu_Initialize(_G[cfg_startpoint], function(self, level) Config_Initialize(self, level, startpoint, func, group) end)
	UIDropDownMenu_SetWidth(_G[cfg_startpoint], 120)
	UIDropDownMenu_SetButtonWidth(_G[cfg_startpoint], 124)
	UIDropDownMenu_JustifyText(_G[cfg_startpoint], "LEFT")
	group.FG_ConfigStartPoint = _G[cfg_startpoint]

--[[
	local cfg_iconmode = CreateFrame("CheckButton", "FB_"..tostring(group).."_ConfigIconMode", group, "UICheckButtonTemplate")
	cfg_iconmode.Text = _G[cfg_iconmode:GetName().."Text"]
	cfg_iconmode.Text:SetText("Enabled")
	cfg_iconmode:SetScript("OnClick", function() group:SetIconMode(cfg_iconmode:GetChecked()) end)
	cfg_iconmode:SetPoint("LEFT", combo, "RIGHT", -15, 0)
	group.FG_ConfigIconMode = cfg_iconmode
]]

	group:SetScript("OnSizeChanged", onResize)
	group:SetScript("OnDragStart", onDragStart)
	group:SetScript("OnDragStop", onDragStop)

	-- import callback functions
	group.callbacks = cbh:New(group)

	group.bars = {}
	group.title = title or ""
	group.name = name
	group:SetVisibleBars(def_MaxBars)
	group:SetColumns(def_Columns)
--	group:SetAutoResize(def_AutoResize)
	group:SetStartPoint(def_StartPoint)
	group.sortfield = def_SortField
	group.sortasc = def_SortAscending
	group:SetIconMode(def_IconMode)
	group.iconsize = def_IconSize

	hooksecurefunc(group, "SetScale", onSetScale)

	ToggleConfig(group)
	group:Lock(true)

	return group
end

--- Rearranges the group's bars.
function groupPrototype:Rearrange()
	if self.bars == nil then return end
	
	local tmp = {}
	for bar in pairs(self.bars) do
		tmp[#tmp+1] = bar
	end
	tsort(tmp, function(a, b) return SortBars(a, b, self.sortfield) end)
	local lastBar = false
	local rowFirst = false
	local barNo = 0

	local barMax
	if self.maxbars>0 and self.maxbars <= #tmp then
		barMax = self.maxbars
	else
		barMax = #tmp
	end

	local startpoint = self.startpoint
	local width = self:GetWidth()
	local columns
	if self.columns == 0 then
		if self.iconmode then
			columns = floor(width/self.iconmode)
		else
			columns = 1
		end
	else
		columns = self.columns
	end

	local x, y, z, bar
	if self.sortasc then
		x = barMax
		y = 1
		z = -1
	else
		x = 1
		y = barMax
		z = 1
	end

	for i=x, y, z do
		bar = tmp[i]
		bar:Show()
		bar:ClearAllPoints()
		if not bar.iconmode then
			bar:SetWidth(width/columns)
		end
		if startpoint == "topleft" then
			if (barNo % columns) == 0 then
				bar:SetPoint("BOTTOMLEFT", rowFirst or self, "TOPLEFT")
				rowFirst = bar
			else
				bar:SetPoint("LEFT", lastBar or self, "RIGHT")
			end
		elseif startpoint == "topright" then
			if (barNo % columns) == 0 then
				bar:SetPoint("BOTTOMRIGHT", rowFirst or self, "TOPRIGHT")
				rowFirst = bar
			else
				bar:SetPoint("RIGHT", lastBar or self, "LEFT")
			end
		elseif startpoint == "bottomright" then
			if (barNo % columns) == 0 then
				bar:SetPoint("TOPRIGHT", rowFirst or self, "BOTTOMRIGHT")
				rowFirst = bar
			else
				bar:SetPoint("RIGHT", lastBar or self, "LEFT")
			end
		else
			if (barNo % columns) == 0 then
				bar:SetPoint("TOPLEFT", rowFirst or self, "BOTTOMLEFT")
				rowFirst = bar
			else
				bar:SetPoint("LEFT", lastBar or self, "RIGHT")
			end
		end
		lastBar = bar
		barNo = barNo + 1
	end

	if self.maxbars>0 and self.maxbars < #tmp then
		for i=self.maxbars+1, #tmp do
			tmp[i]:Hide()
		end
	end
end

function groupPrototype:SetIconMode(size)
	if size == true or size == 1 then size = def_IconSize end
	self.iconmode = size

	local bars = self.bars
	for k in pairs(bars) do
		k:SetIconMode(size)
	end
	self:Rearrange()
	self.callbacks:Fire("FancyBar_GroupIconMode", self, size)
end

--- Adds a bar to the group.
--@param bar Bar object to add.
function groupPrototype:Add(bar)
	if not bar then return end

	self.bars[bar] = true
	bar.group = self
	bar:SetScale(self:GetScale())
	bar:SetIconMode(self.iconmode)
	self:Rearrange()
end

--- Removes a bar from the group.
--@param bar Bar object to remove.
function groupPrototype:Remove(bar)
	if not bar then return end

	self.bars[bar] = nil
	bar.group = nil
	bar:SetScale(1)
	self:Rearrange()
end

--- Stops and clears all bars from the group.
--@param noEvent If true, bars will not fire their "FB_BarStop" event.
function groupPrototype:Clear(noEvent)
	local bars = self.bars

	for k in pairs(bars) do
		k:Stop(noEvent)
		bars[k] = nil
	end
end

--- Searches in the user data (set via the bar's :SetData() function) of the bars assigned to the group, and retrieves the first bar where key = value. Returns nil if not found.
--@param key Key to search.
--@param value Value to search.
function groupPrototype:Find(key, value)
	for k in pairs(self.bars) do
		if k.data[key] and k.data[key] == value then return k end
	end
	return nil
end

--- Stops and removes any test bars running.
function groupPrototype:StopTest()
	for k in pairs(self.bars) do
		k:Stop(true)
	end
	wipe(self.bars)
	self.callbacks:Fire("FancyBar_GroupTestStop", self)
end

--- Shows a number of test bars in the group.
-- @param number The number of bars to show.
-- Default: 50.
function groupPrototype:RunTest(number)
	self:StopTest()

	if not number then number = 50 end

	for i=1, number do
		local bar = lib:NewBar(self, 200, 16)
		local spell = testicons[mrand(1, #testicons)]
		bar:SetLabel("[T] "..GetSpellInfo(spell))
		bar:SetIcon(select(3, GetSpellInfo(spell)))
		bar:SetDuration(mrand(1, 60), 60)
		bar:SetFlash(8, 1, 1)
--		bar:HideStatusBar(true)
		bar:SetGhost(3, nil, "|cFF00FF00Ready", {0.1, 0.1, 0.8, 0.8})
		bar:SetFadeout(1)
		bar:SetSpark(true)
		bar:Start()
	end
	self.callbacks:Fire("FancyBar_GroupTestStart", self)
end

--- Locks/unlocks the group so it can be moved and resized.
-- Register callbacks "FB_GroupMoved" and "FB_GroupResized" to get the new position and size, respectively.
-- @param state The group is to be locked (true) or unlocked (false)
function groupPrototype:Lock(state)
	if state then
		self:StopTest()
		if self.FG_ConfigCombo:IsShown() then
			ToggleConfig(self)
		end

		self:EnableMouse(false)
		self:SetMovable(false)
		self:SetResizable(false)
		self:RegisterForDrag(nil)

		self.FG_Header:Hide()
		self.FG_HeaderText:Hide()
--		self.FG_LockButton:Hide()
		self.FG_ConfigButton:Hide()
		self.FG_TestButton:Hide()
		self.FG_DragArea:Hide()
		self.FG_DragHandle:Hide()
	else
		self:EnableMouse(true)
		self:SetMovable(true)
		self:SetResizable(true)
		self:RegisterForDrag("LeftButton")

		self.FG_Header:Show()
		self.FG_HeaderText:Show()
--		self.FG_LockButton:Show()
		self.FG_ConfigButton:Show()
		self.FG_TestButton:Show()
		self.FG_DragArea:Show()
		self.FG_DragHandle:Show()

	end
	self.callbacks:Fire("FancyBar_GroupLock", self, state)
end

--[[
--- Automatically resize bars to fit the width of the group.
--For example if the group is 200 pixels wide and it has 2 columns, then bars are resized to 100 pixels.
--@param resize True or false.
--Default: true.
function groupPrototype:SetAutoResize(resize)
	self.autoresize = resize
	self.FG_ConfigResize:SetChecked(resize)
	self:Rearrange()
	self.callbacks:Fire("FancyBar_ChangedAutoResize", self, resize)
end
]]

--- Sets the font used for the label text in all the bars in the group.
--@param font Font to apply. Can be a FontObject or a path to the font file.
--@param size Change size of the font.
function groupPrototype:SetBarLabelFont(font, size)
	for k in pairs(self.bars) do
		k:SetLabelFont(font, size)
	end
end

--- Sets the font used for the duration text in all the bars in the group.
--@param font Font to apply. Can be a FontObject or a path to the font file.
--@param size Change size of the font.
function groupPrototype:SetBarDurationFont(font, size)
	for k in pairs(self.bars) do
		k:SetDurationFont(font, size)
	end
end

--- Sets the maximum number of bars showing for this group. If nil, the group shows all bars added to it.
--@param maxBars The number of bars to show at once.
--Default: nil.
function groupPrototype:SetVisibleBars(maxBars)
	if not maxBars then maxBars = 0 end
	self.maxbars = maxBars
	self.FG_ConfigMaxBars:SetValue(maxBars)
	self.FG_ConfigMaxBars.Value:SetText(maxBars)
	self:Rearrange()
	self.callbacks:Fire("FancyBar_GroupVisibleBars", self, maxBars)
end

--- Sets the number of columns this group's bars are arranged into.
--@param columns Number of columns.
--Default: 1.
function groupPrototype:SetColumns(columns)
	if columns and columns > 0 then
		self.columns = columns
	else
		self.columns = 0
	end
	self.FG_ConfigColumns:SetValue(columns)
	self.FG_ConfigColumns.Value:SetText(columns)
	self:Rearrange()
	self.callbacks:Fire("FancyBar_GroupColumns", self, self.columns)
end

--- Sets the sorting method used to sort the group's bars.
--@param field Sorting field used.
--Values: "label" (label text alphabetically), "time" (time remaining), "created" (time the bar is created) or a data variable stored via the bar's SetData function (make sure it's set in all bars, is not nil and of the same type).
--Default: "time"
--@param ascending Arranges bars in ascending order.
--Values: true, false
--Default: false
function groupPrototype:SetSorting(field, ascending)
	self.sortfield = field
	self.sortasc = ascending
	self:Rearrange()
	self.callbacks:Fire("FancyBar_GroupSorting", self, field, ascending)
end

--- Sets the corner to start bars from.
--Values: "bottomleft", "bottomright", "topleft", "topright".
--Default: "bottomleft".
function groupPrototype:SetStartPoint(point)
	if point == "bottomleft" or point == "bottomright" or point == "topleft" or point == "topright" then
		self.startpoint = point
		UIDropDownMenu_SetSelectedValue(self.FG_ConfigStartPoint, point)
		self:Rearrange()
		self.callbacks:Fire("FancyBar_GroupStartPoint", self, point)
	end
end